#!/usr/bin/python

from sso_server import LoginServer
from sso_client import LoginClient

if __name__ == '__main__':

    server = LoginServer('keys')
    client = LoginClient('example.org', 'keys/example.org')

    #
    # Check works together
    #
    together_nonce1, together_request1 = client.create_request()
    together_incoming_nonce1 = server.verify_request('example.org', together_request1)
    # At this point the server should ask for the username and password
    together_token1 = server.make_token('example.org', together_incoming_nonce1, 'rich@kde.org')
    authenticated_email = client.verify_reply(together_token1, together_nonce1)
    print 'You are authenticated as:', authenticated_email

    #
    # Check the request handling
    #

    # Create two almost identical requests so we can see the tokens are very different
    nonce1, request1 = client.create_request()
    print request1

    nonce2, request2 = client.create_request()
    print request2

    # Verify the request
    incoming_nonce1 = server.verify_request('example.org', request1)
    incoming_nonce2 = server.verify_request('example.org', request2)

    if nonce1 != incoming_nonce1:
        print 'ERROR: Nonce 1 extracted incorrectly'
    if nonce2 != incoming_nonce2:
        print 'ERROR: Nonce 2 extracted incorrectly'

    # Check we can't modify the token
    try:
        l = list(request1)
        l[0] = chr(ord(l[1]) ^ 0x1) # Flip one bit
        badrequest1 = ''.join(l)
        server.verify_request('example.org', badrequest1)
    except Exception, e:
        print 'Expected', e
    else:
        print 'ERROR: This should have failed'

    #
    # Check the response handling
    #

    # Create two almost identical responses so we can see the tokens are very different
    # we deliberately reuse the nonce to increase the likelihood of problems
    token1 = server.make_token('example.org', '0123456789abcdef', 'rich@kde.org')
    print token1

    token2 = server.make_token('example.org', '0123456789abcdef', 'sich@kde.org')
    print token2

    # Create extreme case
    token3 = server.make_token('example.org', '0123456789abcdef', '')
    print token3
    token4 = server.make_token('example.org', '0123456789abcdef', '')
    print token4

    # Verify the tokens
    username1 = client.verify_reply(token1, '0123456789abcdef')
    print username1

    username2 = client.verify_reply(token2, '0123456789abcdef')
    print username2

    # Check we can't modify the token
    try:
        l = list(token1)
        l[0] = chr(ord(l[1]) ^ 0x1) # Flip one bit
        badtoken1 = ''.join(l)
        client.verify_reply(badtoken1, '0123456789abcdef')
    except Exception, e:
        print 'Expected', e
    else:
        print 'ERROR: This should have failed'

    # Check we verify the nonce
    try:
        client.verify_reply(token1, 'X123456789abcdef')
    except Exception, e:
        print 'Expected', e
    else:
        print 'ERROR: This should have failed'

    
