from base64 import b64encode, b64decode
import struct

from Crypto.Cipher import AES
from Crypto import Random

from utils import *

class LoginClient:

    def __init__(self, site, key_file):
        self.site = site
        self.key_file = key_file

    def load_key(self):
        '''Returns the key.'''

        f = open(self.key_file)
        try:
            key = f.read()
            return b64decode(key.strip())
        finally:
            f.close()

    def create_request(self):
        # Select the nonce
        random = Random.new()
        nonce = random.read(NONCE_LENGTH)

        # Create the request
        request = nonce
        request = pad(request)

        # Encrypt the request
        key = self.load_key()
        iv = random.read(AES.block_size)

        aes = AES.new(key, AES.MODE_CBC, iv)
        cipher_text = aes.encrypt(request)

        token = iv + cipher_text

        token = append_hmac(key, token)
        token = b64encode(token)

        return nonce, token

    def verify_reply(self, reply, nonce):
        raw_reply = b64decode(reply)

        # Verify the MAC
        key = self.load_key()
        token = verify_hmac(key, raw_reply)

        # Split up the token
        iv = token[:AES.block_size]
        cipher_text = token[AES.block_size:]

        # Decrypt the token
        aes = AES.new(key, AES.MODE_CBC, iv)
        plain_text = aes.decrypt(cipher_text)

        incoming_nonce = plain_text[:NONCE_LENGTH]
        if incoming_nonce != nonce:
            raise Exception('Bad nonce')

        # Get the email from the token
        length = struct.unpack('!I', plain_text[NONCE_LENGTH:NONCE_LENGTH+4])[0]
        email = plain_text[NONCE_LENGTH+4:NONCE_LENGTH+4+length]

        return email
