from base64 import b64encode, b64decode
from os import path
import struct

from Crypto.Cipher import AES
from Crypto import Random

from utils import *

class LoginServer(object):

    def __init__(self, keystore_path):
        self.keystore_path = keystore_path
        self.random = Random.new()
    
    def load_key(self, site):
        '''Returns the key for the specified site.'''

        key_file = path.join(self.keystore_path, site)
        f = open(key_file)
        try:
            key = f.read()
            return b64decode(key.strip())
        finally:
            f.close()

    def make_token(self, site, nonce, username):
        '''
        Returns an authentication token that will authenticate username for
        the specified site. The nonce is provided by the site in order to
        ensure that the token is for the specified request.
        '''

        # Create the confirmation the login is valid
        confirmation = nonce + struct.pack('!I', len(username)) + username
        confirmation = pad(confirmation)

        # Encrypt the confirmation
        key = self.load_key(site)
        iv = self.random.read(AES.block_size)

        aes = AES.new(key, AES.MODE_CBC, iv)
        cipher_text = aes.encrypt(confirmation)

        token = iv + cipher_text
        token = append_hmac(key, token)
        token = b64encode(token)

        return token

    def verify_request(self, site, request):
        raw_request = b64decode(request)

        # Verify the MAC
        key = self.load_key(site)
        token = verify_hmac(key, raw_request)

        # Split up the token
        iv = token[:AES.block_size]
        cipher_text = token[AES.block_size:]

        # Decrypt the token
        aes = AES.new(key, AES.MODE_CBC, iv)
        plain_text = aes.decrypt(cipher_text)

        nonce = plain_text[:NONCE_LENGTH]
        
        return nonce

