from Crypto.Hash import HMAC, SHA
from Crypto.Cipher import AES

NONCE_LENGTH = AES.block_size

def pad(text):
    '''
    Pad the text to AES block size using PKCS5 method.
    '''
    if len(text) % AES.block_size == 0:
        return text

    padding_count = AES.block_size - len(text) % AES.block_size
    padding = chr(padding_count)
    padded = text + (padding * padding_count)
    return padded

def append_hmac(key, text):
    hmac = HMAC.new(key, digestmod=SHA)
    hmac.update(text)
    mac = hmac.digest()

    return text + mac

def verify_hmac(key, message):
    text = message[:-SHA.digest_size]
    mac = message[-SHA.digest_size:]

    hmac = HMAC.new(key, digestmod=SHA)
    hmac.update(text)
    calculated_mac = hmac.digest()

    if mac != calculated_mac:
        raise Exception('Invalid MAC')

    return text
